"""
graphene-jwt-auth library
-----------------------

Several utilities to implement JWT Authentication in Django and Graphene.

:copyright: (c) 2017 by Darwin Herlinmanda Cahyadi
:license: MIT. See LICENSE for more details
"""

__title__ = 'graphene_jwt_auth'
__version__ = '0.0.1'
__author__ = 'Darwin Herlinmanda Cahyadi'
__license__ = 'MIT'
__copyright__ = 'Copyright 2017 Darwin Herlinmanda Cahyadi'
